#!/bin/sh

VERSION="$(cat 'core/server.go' | grep -E '^\s*Version' | sed -e 's|\s*Version = \"||' -e 's|\"\s*$||')" \
    || {
    echo 'Failed to get gophi version!'
    exit 1
}

GOROOT="${HOME}/Projects/github.com/grufwub/go"
PATH="${GOROOT}/bin:${PATH}"

build_for() {
    # Grab build information
    local protocol="$1" archname="$2" toolchain="$3" os="$4" arch="$5"
    shift 5
    if [ "$arch" = 'arm' ]; then
        local armversion="$1"
        shift 1
    fi

    # Generate unique filename
    local filename="build/gophi.${protocol}_${os}_${archname}"

    # Build binary!
    echo "Building for ${os} ${archname} with ${toolchain}..."
    CC="${toolchain}-gcc" CGO_ENABLED=0 GOOS="$os" GOARCH="$arch" GOARM="$armversion" go build -trimpath -o "$filename" "$@" "cmd/${protocol}/main.go" \
        || {
        echo 'Failed!'
        return 1
    }
}

echo "PLEASE BE WARNED THIS SCRIPT IS WRITTEN FOR A VOID LINUX (MUSL) BUILD ENVIRONMENT"
echo "YOUR CC TOOLCHAIN LOCATIONS MAY DIFFER"
echo "IF THE SCRIPT FAILS, CHECK THE OUTPUT OF: ${LOGFILE}"
echo ""

# Clean build directory
rm -rf build/; mkdir -p build/

# Build time :)

# Linux
build_for 'gopher' '386'      'i686-linux-musl'         'linux' '386'     -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' '386'      'i686-linux-musl'         'linux' '386'     -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'amd64'    'x86_64-linux-musl'       'linux' 'amd64'   -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'amd64'    'x86_64-linux-musl'       'linux' 'amd64'   -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5'    'arm-linux-musleabi'      'linux' 'arm' '5' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5'    'arm-linux-musleabi'      'linux' 'arm' '5' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5hf'  'arm-linux-musleabihf'    'linux' 'arm' '5' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5hf'  'arm-linux-musleabihf'    'linux' 'arm' '5' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6'    'arm-linux-musleabi'      'linux' 'arm' '6' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6'    'arm-linux-musleabi'      'linux' 'arm' '6' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6hf'  'arm-linux-musleabihf'    'linux' 'arm' '6' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6hf'  'arm-linux-musleabihf'    'linux' 'arm' '6' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv7lhf' 'armv7l-linux-musleabihf' 'linux' 'arm' '7' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv7lhf' 'armv7l-linux-musleabihf' 'linux' 'arm' '7' -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'arm64'    'aarch64-linux-musl'      'linux' 'arm64'   -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'arm64'    'aarch64-linux-musl'      'linux' 'arm64'   -buildmode 'pie'     -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'mips'     'mips-linux-musl'         'linux' 'mips'    -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'mips'     'mips-linux-musl'         'linux' 'mips'    -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'mipshf'   'mips-linux-muslhf'       'linux' 'mips'    -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'mipshf'   'mips-linux-muslhf'       'linux' 'mips'    -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'mipsle'   'mipsel-linux-musl'       'linux' 'mipsle'  -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'mipsle'   'mipsel-linux-musl'       'linux' 'mipsle'  -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'mipslehf' 'mipsel-linux-muslhf'     'linux' 'mipsle'  -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'mipslehf' 'mipsel-linux-muslhf'     'linux' 'mipsle'  -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'ppc64'    'powerpc64-linux-musl'    'linux' 'ppc64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'ppc64'    'powerpc64-linux-musl'    'linux' 'ppc64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'ppc64le'  'powerpc64le-linux-musl'  'linux' 'ppc64le' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'ppc64le'  'powerpc64le-linux-musl'  'linux' 'ppc64le' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

# Netbsd
build_for 'gopher' '386'      'i686-linux-musl'         'netbsd'  '386'     -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' '386'      'i686-linux-musl'         'netbsd'  '386'     -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'amd64'    'x86_64-linux-musl'       'netbsd'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'amd64'    'x86_64-linux-musl'       'netbsd'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5'    'arm-linux-musleabi'      'netbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5'    'arm-linux-musleabi'      'netbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5hf'  'arm-linux-musleabihf'    'netbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5hf'  'arm-linux-musleabihf'    'netbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6'    'arm-linux-musleabi'      'netbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6'    'arm-linux-musleabi'      'netbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6hf'  'arm-linux-musleabihf'    'netbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6hf'  'arm-linux-musleabihf'    'netbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv7lhf' 'armv7l-linux-musleabihf' 'netbsd'  'arm' '7' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv7lhf' 'armv7l-linux-musleabihf' 'netbsd'  'arm' '7' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'arm64'    'aarch64-linux-musl'      'netbsd'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'arm64'    'aarch64-linux-musl'      'netbsd'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

# Openbsd
build_for 'gopher' '386'      'i686-linux-musl'         'openbsd'  '386'     -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' '386'      'i686-linux-musl'         'openbsd'  '386'     -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'amd64'    'x86_64-linux-musl'       'openbsd'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'amd64'    'x86_64-linux-musl'       'openbsd'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5'    'arm-linux-musleabi'      'openbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5'    'arm-linux-musleabi'      'openbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5hf'  'arm-linux-musleabihf'    'openbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5hf'  'arm-linux-musleabihf'    'openbsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6'    'arm-linux-musleabi'      'openbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6'    'arm-linux-musleabi'      'openbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6hf'  'arm-linux-musleabihf'    'openbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6hf'  'arm-linux-musleabihf'    'openbsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv7lhf' 'armv7l-linux-musleabihf' 'openbsd'  'arm' '7' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv7lhf' 'armv7l-linux-musleabihf' 'openbsd'  'arm' '7' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'arm64'    'aarch64-linux-musl'      'openbsd'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'arm64'    'aarch64-linux-musl'      'openbsd'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

# Freebsd
build_for 'gopher' '386'      'i686-linux-musl'         'freebsd'  '386'     -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' '386'      'i686-linux-musl'         'freebsd'  '386'     -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'amd64'    'x86_64-linux-musl'       'freebsd'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'amd64'    'x86_64-linux-musl'       'freebsd'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5'    'arm-linux-musleabi'      'freebsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5'    'arm-linux-musleabi'      'freebsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv5hf'  'arm-linux-musleabihf'    'freebsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv5hf'  'arm-linux-musleabihf'    'freebsd'  'arm' '5' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6'    'arm-linux-musleabi'      'freebsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6'    'arm-linux-musleabi'      'freebsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv6hf'  'arm-linux-musleabihf'    'freebsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv6hf'  'arm-linux-musleabihf'    'freebsd'  'arm' '6' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'armv7lhf' 'armv7l-linux-musleabihf' 'freebsd'  'arm' '7' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'armv7lhf' 'armv7l-linux-musleabihf' 'freebsd'  'arm' '7' -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'arm64'    'aarch64-linux-musl'      'freebsd'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'arm64'    'aarch64-linux-musl'      'freebsd'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

# Dragonfly
build_for 'gopher' 'amd64'    'x86_64-linux-musl'       'dragonfly'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'amd64'    'x86_64-linux-musl'       'dragonfly'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

# Macos
build_for 'gopher' 'amd64'    'x86_64-linux-musl'       'darwin'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'amd64'    'x86_64-linux-musl'       'darwin'  'amd64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'

build_for 'gopher' 'arm64'    'aarch64-linux-musl'      'darwin'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
build_for 'gemini' 'arm64'    'aarch64-linux-musl'      'darwin'  'arm64'   -buildmode 'default' -a -tags 'netgo osusergo static_build' -ldflags '-s -w -extldflags "-static"'
