package core

import (
	"net"
	"strconv"
)

// Client holds onto an open Conn to a client, along with connection information
type Client struct {
	conn *conn
	ip   string
	port string
}

// NewClient returns a new client based on supplied net.TCPConn
func NewClient(conn net.Conn) *Client {
	addr, _ := conn.RemoteAddr().(*net.TCPAddr)
	ip, port := addr.IP.String(), strconv.Itoa(addr.Port)
	return &Client{
		conn: wrapConn(conn),
		ip:   ip,
		port: port,
	}
}

// Conn returns the underlying conn
func (c *Client) Conn() *conn {
	return c.conn
}

// IP returns the client's IP string
func (c *Client) IP() string {
	return c.ip
}

// Port returns the client's connected port
func (c *Client) Port() string {
	return c.port
}

// LogInfo logs to the global access logger with the client IP as a prefix
func (c *Client) LogInfo(fmt string, args ...interface{}) {
	AccessLog.Infof("("+c.ip+") "+fmt, args...)
}

// LogError logs to the global access logger with the client IP as a prefix
func (c *Client) LogError(fmt string, args ...interface{}) {
	AccessLog.Errorf("("+c.ip+") "+fmt, args...)
}
