package core

import (
	"net"

	"github.com/grufwub/go-errors"
)

// Listener wraps a net.Listener to return our own clients on each Accept()
type Listener struct {
	l net.Listener
}

// NewListener returns a new Listener object wrapping a net.Listener
func NewListener(l net.Listener) *Listener {
	return &Listener{l}
}

// Accept accepts a new connection and returns a client, or error
func (l *Listener) Accept() (*Client, errors.Error) {
	conn, err := l.l.Accept()
	if err != nil {
		return nil, ErrListenerAccept.Wrap(err)
	}
	return NewClient(conn), nil
}
