package core

// Request is a data structure for storing a filesystem path, and query, parsed from a client's request
type Request struct {
	path  *Path
	query string
}

// NewRequest returns a new Request object
func NewRequest(path *Path, query string) *Request {
	return &Request{
		path:  path,
		query: query,
	}
}

// String returns the full parsed request string (mainly for logging)
func (r *Request) String() string {
	query := r.query
	if query != "" {
		query = "?" + query
	}
	return r.path.Selector() + query
}

// Path returns the requests associate Path object
func (r *Request) Path() *Path {
	return r.path
}

// Query returns the request's query string
func (r *Request) Query() string {
	return r.query
}

// AddToQuery adds provided query to beginning of existing query, formatting as necessary
func (r *Request) AddToQuery(query string) {
	// Ensure we have been given query
	if len(query) < 1 {
		return
	}

	// Either append or set query
	if len(r.query) > 0 {
		r.query = query + "&" + r.query
	} else {
		r.query = query
	}
}

// Remap modifies a request to use new relative path, and accommodate supplied extra query if provided
func (r *Request) Remap(rel, query string) {
	r.AddToQuery(query)
	r.path.Remap(rel)
}

// RemapDirect modifies a request to use new absolute path, and accomodate supplied extra query if provided
func (r *Request) RemapDirect(abs, query string) {
	r.AddToQuery(query)
	r.path.RemapDirect(abs)
}
