package gemini

import (
	"gophi/core"

	"github.com/grufwub/go-errors"
)

// Gemini specific base errors
var (
	errInvalidTLSConfig = errors.New(invalidTLSConfigErrStr)
	errInvalidProtocol  = errors.New(inavlidProtocolErrStr)
	errInvalidHostPort  = errors.New(invalidHostPortErrStr)
)

// generateErrorResponse takes an error code and generates an error response byte slice
func generateErrorResponse(err errors.Error) ([]byte, bool) {
	switch {
	case err.Equals(core.ErrConnWrite):
		return nil, false // no point responding if we couldn't write
	case err.Equals(core.ErrConnRead):
		return buildErrorResponse("40", statusMeta40), true
	case err.Equals(core.ErrConnClose):
		return nil, false // no point responding if we couldn't close
	case err.Equals(core.ErrMutexUpgrade):
		return buildErrorResponse("40", statusMeta40), true
	case err.Equals(core.ErrMutexDowngrade):
		return buildErrorResponse("40", statusMeta40), true
	case err.Equals(core.ErrFileOpen):
		return buildErrorResponse("51", statusMeta51), true
	case err.Equals(core.ErrFileStat):
		return buildErrorResponse("51", statusMeta51), true
	case err.Equals(core.ErrFileRead):
		return buildErrorResponse("51", statusMeta51), true
	case err.Equals(core.ErrFileType):
		return buildErrorResponse("51", statusMeta51), true
	case err.Equals(core.ErrDirectoryRead):
		return buildErrorResponse("51", statusMeta51), true
	case err.Equals(core.ErrRestrictedPath):
		return buildErrorResponse("51", statusMeta51), true
	case err.Equals(core.ErrInvalidRequest):
		return buildErrorResponse("59", statusMeta59), true
	case err.Equals(core.ErrCGIStart):
		return buildErrorResponse("42", statusMeta42), true
	case err.Equals(core.ErrCGIExitCode):
		return buildErrorResponse("42", statusMeta42), true
	case err.Equals(errInvalidProtocol):
		return buildErrorResponse("53", statusMeta59), true
	case err.Equals(errInvalidHostPort):
		return buildErrorResponse("53", statusMeta53), true
	default:
		return nil, false
	}
}

func buildErrorResponse(statusCode, statusMeta string) []byte {
	return buildResponseHeader(statusCode, statusMeta)
}
