package gopher

import (
	"gophi/core"

	"github.com/grufwub/go-errors"
)

// Gopher specific error codes
var (
	errInvalidGophermap  = errors.New(invalidGophermapErrStr)
	errSubgophermapIsDir = errors.New(subgophermapIsDirErrStr)
	errSubgophermapSize  = errors.New(subgophermapSizeErrStr)
)

// generateErrorResponse takes an error code and generates an error response byte slice
func generateErrorResponse(err errors.Error) ([]byte, bool) {
	switch {
	case err.Equals(core.ErrConnWrite):
		return nil, false // no point responding if we couldn't write
	case err.Equals(core.ErrConnRead):
		return buildErrorLine(errorResponse503), true
	case err.Equals(core.ErrConnClose):
		return nil, false // no point responding if we couldn't close
	case err.Equals(core.ErrMutexUpgrade):
		return buildErrorLine(errorResponse500), true
	case err.Equals(core.ErrMutexDowngrade):
		return buildErrorLine(errorResponse500), true
	case err.Equals(core.ErrFileOpen):
		return buildErrorLine(errorResponse404), true
	case err.Equals(core.ErrFileStat):
		return buildErrorLine(errorResponse500), true
	case err.Equals(core.ErrFileRead):
		return buildErrorLine(errorResponse500), true
	case err.Equals(core.ErrFileType):
		return buildErrorLine(errorResponse404), true
	case err.Equals(core.ErrDirectoryRead):
		return buildErrorLine(errorResponse500), true
	case err.Equals(core.ErrRestrictedPath):
		return buildErrorLine(errorResponse403), true
	case err.Equals(core.ErrInvalidRequest):
		return buildErrorLine(errorResponse400), true
	case err.Equals(core.ErrCGIStart):
		return buildErrorLine(errorResponse500), true
	case err.Equals(core.ErrCGIExitCode):
		return buildErrorLine(errorResponse500), true
	case err.Equals(errInvalidGophermap):
		return buildErrorLine(errorResponse500), true
	case err.Equals(errSubgophermapIsDir):
		return buildErrorLine(errorResponse500), true
	case err.Equals(errSubgophermapSize):
		return buildErrorLine(errorResponse500), true
	default:
		return nil, false
	}
}
