package gopher

import (
	"gophi/core"
	"net"

	"github.com/grufwub/go-config"
	"github.com/grufwub/go-errors"
	"github.com/grufwub/go-filecache"
)

// Run does as says :)
func Run() {
	// Create new TOML config parser
	tree := make(config.Tree)

	// Parse gopher specific flags, then all
	pWidth := tree.Uint64("gopher.page-width", 80)
	footerText := tree.String("gopher.footer", "")
	subgopherSizeMax := tree.Float64("gopher.subgopher-max", 1.0)
	admin := tree.String("gopher.admin-email", "")
	desc := tree.String("gopher.description", "")
	geo := tree.String("gopher.geolocation", "")
	core.ParseConfigAndSetup(
		tree,
		"gopher",
		70,
		func() (*core.Listener, errors.Error) {
			l, err := net.Listen("tcp", core.Bind+":"+core.Port)
			if err != nil {
				return nil, core.ErrListenerBegin.Wrap(err)
			}
			return core.NewListener(l), nil
		},
		newFileContent,
		handleDirectory,
		handleLargeFile,
		appendCgiEnv,
	)

	// Setup gopher specific global variables
	subgophermapSizeMax = int64(1048576.0 * *subgopherSizeMax) // convert float to megabytes
	pageWidth = int(*pWidth)
	footer = buildFooter(*footerText)

	// Add generated policy file to cache
	p := core.NewSanitizedPathAtRoot(core.Root, "caps.txt")
	core.SystemLog.Infof("Generating policy file %s...", p.Absolute())
	core.FileCache.Put(filecache.NewFile(p.Absolute(), false, &generatedFileContent{generateCapsTxt(*desc, *admin, *geo)}))

	// Remove things we don't need hanging around
	desc = nil
	admin = nil
	geo = nil
	tree = nil

	// Start!
	core.Start(serve)
}
